package main

import (
	"query-service/internal/adapters/brokeradapter"
	"query-service/internal/drivers/brokerdriver"
	"query-service/internal/drivers/keyvaluedriver"
	"query-service/internal/drivers/rpcdriver"
	"query-service/internal/usecases/consume"
	"query-service/internal/usecases/databaseinfo"
	"query-service/internal/usecases/produce"
	"query-service/internal/usecases/request"
	"query-service/pkg/logger"

	"git.science.uu.nl/datastrophe/query-conversion/aql"
	"github.com/thijsheijden/alice"
)

func main() {
	alice.SetLogging()
	logger.Start()

	// MARK: Create relevant services
	redisService := keyvaluedriver.NewRedisDriver()

	// Create new rpc driver
	rpcDriver := rpcdriver.New()

	// MARK: Create alice RabbitMQ services
	brokerGateway := brokeradapter.CreateGateway()
	aliceBroker := brokerdriver.CreateAliceBroker(brokerGateway)

	// Instantiate an implementation of the produce UseCase
	produceService := produce.NewService(aliceBroker, redisService)

	// MARK: Create relevant services for consuming a message
	convertQueryService := aql.NewService()

	requestSenderService := request.NewService()

	databaseInfoService := databaseinfo.NewService(rpcDriver)

	consumeService := consume.NewService(aliceBroker, produceService, convertQueryService, requestSenderService, databaseInfoService)

	// MARK: Start services
	redisService.Start()
	produceService.Start()

	go consumeService.Start()

	select {}
}
